/*
∗ Language: Privas
∗ Processor
*/

grammar Privas;

@header{
    import java.util.*;
    import org.privas.data.*;
    import org.privas.techniques.*;
    import org.privas.parser.*;
}

@lexer::members {
  private boolean keepNewLine = false;
}

@parser::members {
    private void addNode(Generalization generalizationParent, GeneralizationNode nodeParent, GeneralizationNode node, int level, int numberOfTabs) {
        GeneralizationNode correctParent = nodeParent;

        if (numberOfTabs == 1) {
            generalizationParent.add(node);
        }
        else if (level == numberOfTabs) {
            nodeParent.add(node);
        }
        else if (level > numberOfTabs) {
            int numberOfTabsToGetToCorrectParent = level - numberOfTabs;

            while (numberOfTabsToGetToCorrectParent > 0) {
                correctParent = nodeParent.getParent();
                numberOfTabsToGetToCorrectParent--;
            }

            correctParent.add(node);
            node.setParent(correctParent);
        }
        else {
            // error
        }
    }

    private static Range rangeFrom(Token t1, Token t2, boolean leftInclusive, boolean rightInclusive) {
        Double n1 = t1 != null ? Double.valueOf(t1.getText()) : null;
        Double n2 = t2 != null ? Double.valueOf(t2.getText()) : null;

        return new Range(n1, n2, leftInclusive, rightInclusive);
    }
}

privas returns[PrivacyDefinition privacyDefinition]
@init{
   $privas.privacyDefinition = new PrivacyDefinition();
   List<TableToPrivateDetails> tableList = new ArrayList<TableToPrivateDetails>();
   String name;
}
@after{
    $privas.privacyDefinition.calculateErrors();
}
        : repositoryOptions[$privas.privacyDefinition] dataDescription[$privas.privacyDefinition] privacyOptions[$privas.privacyDefinition] (generalizationTrees[$privas.privacyDefinition])?
        ;


repositoryOptions [PrivacyDefinition privacyDefinition]
        : 'Relational' relationalRepositoryOptions      {   $repositoryOptions.privacyDefinition.setRepositoryType(RepositoryType.MYSQL);
                                                            $repositoryOptions.privacyDefinition.setConnectionString($relationalRepositoryOptions.value);
                                                        }
        | 'CSV' csvRepositoryOptions                    {   $repositoryOptions.privacyDefinition.setRepositoryType(RepositoryType.CSV);
                                                            $repositoryOptions.privacyDefinition.setPath($csvRepositoryOptions.value);
                                                        }
        ;


relationalRepositoryOptions returns[String value]
        : 'Connection String:' STRING_LITERAL           {   $relationalRepositoryOptions.value = $STRING_LITERAL.text; }
        ;

csvRepositoryOptions returns[String value]
        : 'Path:' STRING_LITERAL                        {   $csvRepositoryOptions.value = $STRING_LITERAL.text; }
        ;

dataDescription [PrivacyDefinition privacyDefinition]
@init{
   List<TableToPrivateDetails> tableList = new ArrayList<TableToPrivateDetails>();
   String name;
}
@after{
    $dataDescription.privacyDefinition.setTableToPrivateDetails(tableList);
    $dataDescription.privacyDefinition.setName(name);
}
    : NAME entities[tableList]          {  name = $NAME.text;
                                           tableList = $entities.tablesOut;
                                        }
    ;


entities [List<TableToPrivateDetails> tablesIn]
    returns [List<TableToPrivateDetails> tablesOut]
@after{
     $entities.tablesOut = $entities.tablesIn;
}
    : entity   { $entities.tablesIn.add($entity.tablePrivateDetailsOut); }
    (entity    { $entities.tablesIn.add($entity.tablePrivateDetailsOut); }
    )*
    ;


entity
    returns[TableToPrivateDetails tablePrivateDetailsOut]
@init{
    TableToPrivateDetails tablePrivateDetails = new TableToPrivateDetails();
}
@after{
    $entity.tablePrivateDetailsOut = tablePrivateDetails;
}
    : NAME '[' attributeList ']' { tablePrivateDetails.setTableName($NAME.text);
                                   tablePrivateDetails.setAttributesOptions($attributeList.attributesOptionsOut);
                                 }
    ;


attributeList returns[AttributesOptions attributesOptionsOut]
@init{
    AttributesOptions attributesOptions = new AttributesOptions();
}
@after{
    $attributeList.attributesOptionsOut = attributesOptions;
}
    : attribute[attributesOptions]     { attributesOptions = $attribute.attributesOptionsOut; }
    (',' attribute[attributesOptions]  { attributesOptions = $attribute.attributesOptionsOut; }
    )*
    ;

attribute [AttributesOptions attributesOptionsIn]
    returns [AttributesOptions attributesOptionsOut]
@after{
    $attribute.attributesOptionsOut = $attribute.attributesOptionsIn;
}
    : attributeValue attributeTree? {   Attribute attribute = $attributeValue.attributeOut;
                                        if ($attributeTree.ctx != null) {
                                            attribute.setType($attributeTree.typeName);
                                        }
                                        $attribute.attributesOptionsIn.addAttribute(attribute);
                                    }
    ;

attributeValue
    returns [Attribute attributeOut]
@init {
    boolean isToPrivate = false;
    boolean isToHide = false;
    boolean isId = false;
    boolean isQId = false;
    boolean isSA = false;
    String name = null;
}
@after{
    $attributeValue.attributeOut = new Attribute(name, isId, isQId, isSA);
}
    : '*' NAME  { name = $NAME.text; isToPrivate = true; }
    | '^' NAME  { name = $NAME.text; isToHide = true; }
    | '@' NAME  { name = $NAME.text; isId = true; }
    | '&' NAME  { name = $NAME.text; isQId = true; }
    | '~' NAME  { name = $NAME.text; isSA = true; }
    | NAME      { name = $NAME.text; }
    ;

attributeTree
    returns [String typeName]
    : '::' NAME { $attributeTree.typeName = $NAME.text; }
    ;

privacyOptions [PrivacyDefinition privacyDefinition]
    : 'Prevent from:' '[' attackModelList ']'       { $privacyOptions.privacyDefinition.setAttackModels($attackModelList.models); }
    ;

attackModelList
    returns[List<AttackModel> models]
@init{
    $attackModelList.models = new ArrayList<AttackModel>();
}
@after{
    if (!ValidatePrivacyOptions.validate($attackModelList.models)) {
        List<List<AttackModel>> possible = ValidatePrivacyOptions.getPossibleValues($attackModelList.models);
        System.out.println(possible);

        $attackModelList.models = new ArrayList<AttackModel>();
    }
}
    : attackModel               { $attackModelList.models.add($attackModel.model); }
    ( ',' attackModel           { $attackModelList.models.add($attackModel.model); }
    )*
    ;

attackModel
    returns [ AttackModel model ]
    : attackModelName attackModelParameters? { $attackModel.model = $attackModelName.model; }
    ;

attackModelName
    returns [ AttackModel model ]
    : 'recordLinkage'               { $attackModelName.model = AttackModel.RECORD_LINKAGE; }
    | 'attributeLinkage'            { $attackModelName.model = AttackModel.ATTRIBUTE_LINKAGE; }
    | 'tableLinkage'                { $attackModelName.model = AttackModel.TABLE_LINKAGE; }
    | 'probabilisticAttack'         { $attackModelName.model = AttackModel.PROBABILISTIC_ATTACK; }
    ;

attackModelParameters
    : '<' NAME '=' NUM (',' NAME '=' NUM)* '>'
    ;

generalizationTrees [PrivacyDefinition privacyDefinition]
    : generalizationTreeHeader           { $generalizationTrees.privacyDefinition.addGeneralization($generalizationTreeHeader.generalization); }
      ( '\n' generalizationTreeHeader    { $generalizationTrees.privacyDefinition.addGeneralization($generalizationTreeHeader.generalization); }
      )*
    ;

generalizationTreeHeader
    returns [ Generalization generalization ]
@init {
    String alias = null;
    $generalizationTreeHeader.generalization = null;
}
    : PLUS name=NAME ('as' a=NAME { alias = $a.text; })? '\n' {  if (alias != null) {
                                                                    $generalizationTreeHeader.generalization = new Generalization($name.text, alias);
                                                                 } else {
                                                                    $generalizationTreeHeader.generalization = new Generalization($name.text);
                                                                 }
                                                              }
                            (generalizationTree[$generalizationTreeHeader.generalization, null, 1] ('\n'|EOF)
                            )+
    ;

generalizationTree[Generalization generalization, GeneralizationNode nodeParent, int level]
@init {
    int nextLevel = level+1;
    int tabs = 0;
    GeneralizationNode node = null;
}
@after {
    addNode(generalization, nodeParent, node, level, tabs);
}
    : (TAB { tabs++; })* '+' NAME '\n'                             { node = new GeneralizationNode(nodeParent, $NAME.text); }
               (c=generalizationTree[generalization, node, nextLevel] ('\n'|EOF)
               )+
    | (TAB { tabs++; })* '>' STRING_LITERAL ':' setOfValues         { node = new GeneralizationNodeSetValues(nodeParent, $STRING_LITERAL.text); }
    | (TAB { tabs++; })* '>' NAME ':' setOfValues                   { node = new GeneralizationNodeSetValues(nodeParent, $NAME.text); }
    | (TAB { tabs++; })* '>' STRING_LITERAL ':' range               { node = new GeneralizationNodeRange(nodeParent, $STRING_LITERAL.text, $range.value); }
    | (TAB { tabs++; })* '>' NAME ':' range                         { node = new GeneralizationNodeRange(nodeParent, $NAME.text, $range.value); }
    ;

setOfValues
    : '{' STRING_LITERAL (',' STRING_LITERAL)* '}'
    ;

range
    returns [ Range value ]
    : ']' n1=NUM? ',' n2=NUM? '['         { $range.value = rangeFrom($n1, $n2, false, false); }
    | ']' n1=NUM? ',' n2=NUM? ']'         { $range.value = rangeFrom($n1, $n2, false, true); }
    | '[' n1=NUM? ',' n2=NUM? '['         { $range.value = rangeFrom($n1, $n2, true, false); }
    | '[' n1=NUM? ',' n2=NUM? ']'         { $range.value = rangeFrom($n1, $n2, true, true); }
    ;

/* Lexic analyzer definition*/

PLUS
    : '+'  {keepNewLine = true;}
    ;

NAME
    : [a-zA-Z][-_a-zA-Z0-9]*
    ;

NUM : [0-9]+
    | [0-9]+ '.' [0-9]*
    ;

STRING_LITERAL
    : '"' (~["\\\r\n])* '"'
    | '\'' (~['\\\r\n])* '\''
    ;

TAB   : ('\t'|'    '|'  ') { if (!keepNewLine) skip(); };
SPACE : ' ' -> skip;
ENTER : '\n' { if (!keepNewLine) skip(); };
WINDOWSENTER : '\r\n' { if (!keepNewLine) skip(); };
