package org.privas.data;

import com.opencsv.CSVWriter;
import org.privas.parser.PrivacyDefinition;

import java.io.FileWriter;
import java.io.IOException;
import java.util.stream.Collectors;

public class CSVExporter implements Exporter<TableData> {

    private String path;

    public CSVExporter(PrivacyDefinition privacyDefinition) {
        this.path = privacyDefinition.getPath().replace(".csv", "_privas.csv");
    }

    @Override
    public void init() {

    }

    @Override
    public void export(TableData data) {
        writeCSV(data);
    }

    private void writeCSV(TableData tableData) {
        try {
            CSVWriter writer = new CSVWriter(new FileWriter(path),
                    CSVWriter.DEFAULT_SEPARATOR,
                    CSVWriter.NO_QUOTE_CHARACTER,
                    CSVWriter.DEFAULT_ESCAPE_CHARACTER,
                    CSVWriter.RFC4180_LINE_END);

            String[] headers = tableData.getMetaInfo().getColumnsInfo().stream()
                    .map(ColumnMetaData::getName)
                    .toArray(String[]::new);

            writer.writeNext(headers);

            for (int i = 0; i < tableData.getNumberOfRows(); i++) {
                Row row = tableData.getCurrentRow(i);

                String[] values = row.getColumns().toArray(new String[0]);

                writer.writeNext(values);
            }

            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
