package org.privas.data;

import org.privas.parser.PrivacyDefinition;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.stream.Collectors;

public class MySQLExporter implements Exporter<TableData> {

    private String dbNamePrivas;
    private Connection connection;
    private PrivacyDefinition privacyDefinition;

    public MySQLExporter(PrivacyDefinition privacyDefinition) {
        this.privacyDefinition = privacyDefinition;
        connect(privacyDefinition.getConnectionString());
        dbNamePrivas = privacyDefinition.getName() + "_privas";
    }

    @Override
    public void init() {
        dropDatabase(dbNamePrivas);
        createDatabase(dbNamePrivas);
    }

    @Override
    public void export(TableData data) {
        MetaInfo tableMetaInfo = data.getMetaInfo();
        createTable(dbNamePrivas, data.getTableName(), tableMetaInfo.getColumnsInfo());


        for (int i = 0; i < data.getNumberOfRows(); i++) {
            Row row = data.getOriginalRow(i);

            insertData(dbNamePrivas, data.getTableName(), row);
        }
    }

    private void createTable(String dbName, String tableName, List<ColumnMetaData> columns) {
        try {
            Statement stmt = connection.createStatement();
            List<String> columnsString = columns.stream().map(e -> e.toString()).collect(Collectors.toList());

            String tableParams = String.join(",", columnsString);

            String teste = "CREATE TABLE " + dbName + "." + tableName +"(" + tableParams + ")" ;
            //System.out.println(teste);

            stmt.executeUpdate("CREATE TABLE " + dbName + "." + tableName +"(" + tableParams + ");");
        } catch (SQLException e) {
            // error
            System.out.println(e);
        }
    }

    private void insertData(String dbName, String tableName, Row row) {
        try {
            Statement stmt = connection.createStatement();

            String valueString = row.convertToSQL();

            String teste = "INSERT INTO " + dbName + "." + tableName + " VALUES " + valueString;
            //System.out.println("SQL_INSERT= " + teste);

            stmt.executeUpdate("INSERT INTO " + dbName + "." + tableName + " VALUES " + valueString);
        } catch (SQLException e) {
            // error
            System.out.println(e);
        }
    }

    private void dropDatabase(String databaseName) {
        try {
            Statement stmt = connection.createStatement();

            stmt.executeUpdate("DROP DATABASE IF EXISTS " + databaseName);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private void createDatabase(String databaseName) {
        try {
            Statement stmt = connection.createStatement();

            stmt.executeUpdate("CREATE DATABASE " + databaseName);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private void connect(String connectionString) {
        try {
            //connectionString = "jdbc:mysql://localhost/?user=root&password=simplihire";

            connection = DriverManager.getConnection(connectionString);
        } catch (SQLException e) {
            // error
        }
    }
}
