package org.privas.data;

import org.privas.parser.AttributesOptions;
import org.privas.parser.PrivacyDefinition;
import org.privas.parser.TableToPrivateDetails;

import java.sql.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class MySQLExtractor implements Extractor<TableData> {
    private Connection connection;

    public MySQLExtractor(PrivacyDefinition privacyDefinition) {
        connect(privacyDefinition.getConnectionString());
    }

    @Override
    public Collection<TableData> extract(PrivacyDefinition privacyDefinition) {
        List<TableData> data = new ArrayList<>();

        // para cada tabela:
        for (TableToPrivateDetails tableDetail : privacyDefinition.getTableToPrivateDetails()) {
            // ler tabela
            TableData tableData = getTable(privacyDefinition.getName(), tableDetail.getTableName(), tableDetail.getAttributesOptions(), privacyDefinition);

            data.add(tableData);
        }

        return data;
    }

    private TableData getTable(String databaseName, String name, AttributesOptions attributesOptions, PrivacyDefinition privacyDefinition) {
        String query = "SELECT * FROM " + databaseName + "." + name;

        Statement stmt = null;
        ResultSet rs = null;

        TableData tableData = null;

        try {
            // Do something with the Connection
            stmt = connection.createStatement();
            rs = stmt.executeQuery(query);

            // Now do something with the ResultSet ....
            ResultSetMetaData rsmd = rs.getMetaData();
            int columnsNumber = rsmd.getColumnCount();

            String tableName = rsmd.getTableName(1);

            tableData = new TableData(tableName, attributesOptions, privacyDefinition.getGeneralizations());

            for (int i = 1; i <= columnsNumber; i++) {
                String columnName = rsmd.getColumnLabel(i);
                String dataTypeLabel = rsmd.getColumnTypeName(i);
                int dataType = rsmd.getColumnType(i);

                int length = rsmd.getColumnDisplaySize(i);

                boolean nullable = rsmd.isNullable(i) == ResultSetMetaData.columnNullable;

                tableData.addColumnName(columnName, dataTypeLabel, dataType, length, i - 1, nullable);
            }

            tableData.consolidateColumns();

            while (rs.next()) {
                Row row = new Row();

                for (int i = 1; i <= columnsNumber; i++) {
                    String value = rs.getString(i);

                    row.addValue(value);
                }

                tableData.add("", row);
            }
        } catch (SQLException e) {
            // error
            System.out.println(e);
        }

        return tableData;
    }

    private void connect(String connectionString) {
        try {
            //connectionString = "jdbc:mysql://localhost/?user=root&password=simplihire";

            connection = DriverManager.getConnection(connectionString);
        } catch (SQLException e) {
            // error
        }
    }
}
