package org.privas.data;

import org.privas.parser.AttributesOptions;
import org.privas.parser.Generalization;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class TableData implements Data<Row> {

    private String tableName;
    private ArrayList<Row> data;
    private MetaInfo metaInfo;
    private AttributesOptions attributesOptions;

    private ArrayList<String> qids;
    private ArrayList<String> normal;

    private Map<String, Generalization> generalizations;

    public TableData(String tableName, AttributesOptions attributesOptions, Map<String, Generalization> generalizations) {
        this.tableName = tableName;
        this.data = new ArrayList<>();
        this.attributesOptions = attributesOptions;
        this.metaInfo = new MetaInfo();
        this.qids = new ArrayList<>();
        this.normal = new ArrayList<>();
        this.generalizations = generalizations;
        this.data = new ArrayList<>();
    }

    public TableData(TableData tableData) {
        this.tableName = tableData.getTableName();
        this.attributesOptions = tableData.getAttributesOptions();
        this.generalizations = tableData.getGeneralizations();
        this.metaInfo = tableData.getMetaInfo();
        this.data = new ArrayList<>();
    }

    public void addColumnName(String columnLabel, String dataTypeLabel, int dataType, int length, int originalPosition, boolean nullable) {
        boolean isQId = attributesOptions.isQIdAttribute(columnLabel);
        boolean isSA = attributesOptions.isSAAttribute(columnLabel);

        if (attributesOptions.isQIdAttribute(columnLabel)) {
            this.qids.add(columnLabel);
        }
        else {
            this.normal.add(columnLabel);
        }

        this.metaInfo.addColumnName(columnLabel, dataTypeLabel, dataType, length, originalPosition, nullable, isQId, isSA, attributesOptions.getAttribute(columnLabel));
    }

    public void consolidateColumns() {
        int number = 0;

        for (String qid : qids) {
            this.metaInfo.setCurrentPosition(qid, number);
            number++;
        }

        for (String n : normal) {
            this.metaInfo.setCurrentPosition(n, number);
            number++;
        }
    }

    @Override
    public void add(String id, Row data) {
        Row dataSorted = sortRowByCurrent(data);

        this.data.add(dataSorted);
    }

    public void addRaw(Row data) {
        // adds without sorting
        this.data.add(data);
    }

    @Override
    public Row get(String id) {
        return null;
    }

    @Override
    public String getMainId() {
        return tableName;
    }

    public String getTableName() {
        return tableName;
    }

    public MetaInfo getMetaInfo() {
        return metaInfo;
    }

    public void setMetaInfo(MetaInfo metaInfo) {
        this.metaInfo = metaInfo;
    }

    public Map<String, Generalization> getGeneralizations() {
        return generalizations;
    }

    public int getNumberOfRows() {
        return data.size();
    }

    public Row getCurrentRow(int number) {
        return data.get(number);
    }

    public Row getOriginalRow(int number) {
        Row dataSorted = sortRowByOriginal(data.get(number));

        return dataSorted;
    }

    public AttributesOptions getAttributesOptions() {
        return attributesOptions;
    }

    public int getQIdsNumber() {
        return this.qids.size();
    }

    public List<Generalization> getGeneralizationsForColumns() {
        List<Generalization> generalizations = new ArrayList<>();

        this.metaInfo.getColumnsInfo().forEach(columnMetaData -> {
            if (columnMetaData.isToPrivate()) {
                Generalization generalization = getGeneralizationForColumn(columnMetaData);

                if (generalization != null) {
                    generalizations.add(generalization);
                }
            }
        });

        return generalizations;
    }

    public List<Integer> getSAPos() {
        List<Integer> pos = new ArrayList<>();

        for (String sa: attributesOptions.getSensitiveAttributes()) {
            int columnNumber = metaInfo.getColumnNum(sa);

            pos.add(metaInfo.getCurrentColumnPosition(columnNumber));
        }

        return pos;
    }

    public List<Integer> getIdsPos() {
        List<Integer> pos = new ArrayList<>();

        for (String id: attributesOptions.getIdAttributes()) {
            int columnNumber = metaInfo.getColumnNum(id);

            pos.add(metaInfo.getCurrentColumnPosition(columnNumber));
        }

        return pos;
    }

    private Row sortRowByCurrent(Row row) {
        ArrayList<String> rowData = row.getColumns();
        int numberOfElements = rowData.size();
        Row rowSorted = new Row(numberOfElements);

        for (int i = 0; i < numberOfElements; i++) {
            int dataPosition = this.metaInfo.getCurrentColumnPosition(i);

            rowSorted.addValue(dataPosition, rowData.get(i));
        }

        return rowSorted;
    }

    private Row sortRowByOriginal(Row row) {
        ArrayList<String> rowData = row.getColumns();
        int numberOfElements = rowData.size();
        Row rowSorted = new Row(numberOfElements);

        for (int i = 0; i < numberOfElements; i++) {
            int dataPosition = this.metaInfo.getOriginalColumnPosition(i);

            rowSorted.addValue(dataPosition, rowData.get(i));
        }

        return rowSorted;
    }

    private Generalization getGeneralizationForColumn(ColumnMetaData column) {
        if (column.isRangeType()) {
             return column.getGeneralizationRange(data);
        }

        return this.generalizations.get(column.getAttribute().getType());
    }
}
