package org.privas.parser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class Generalization {

    private String name;
    private String alias;
    private List<GeneralizationNode> generalizationNodes;

    public Generalization(String name) {
        this.name = name;
        generalizationNodes = new ArrayList<>();
    }

    public Generalization(String name, String alias) {
        this(name);
        this.alias = alias;
    }

    public void add(GeneralizationNode node) {
        generalizationNodes.add(node);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAlias() {
        return alias;
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    public List<GeneralizationNode> getGeneralizationNodes() {
        return generalizationNodes;
    }

    public void setGeneralizationNodes(List<GeneralizationNode> generalizationNodes) {
        this.generalizationNodes = generalizationNodes;
    }

    public int getNumberOfLeafsCovered() {
        return generalizationNodes.stream()
                .map(GeneralizationNode::getNumberOfLeafsCovered)
                .flatMapToInt(IntStream::of)
                .sum();
    }

    public List<GeneralizationNode> getAllNodes() {
        return generalizationNodes.stream()
                .map(GeneralizationNode::getAllNodes)
                .flatMap(Collection::stream)
                .collect(Collectors.toList());
    }

    public boolean isRange() {
        return false;
    }

    public GeneralizationNode getGeneralizationFor(String value) {
        if (name.equals(value)) {
            return buildNodeFromThis();
        }

        Optional<GeneralizationNode> node = generalizationNodes.stream()
                .map(n -> n.getGeneralizationFor(value))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .findFirst();

        return node.orElse(buildNodeFromThis());
    }

    public int getNrGeneralizationsAvailable() {
        return 1 + generalizationNodes.stream().mapToInt(GeneralizationNode::height).max().orElse(0);
    }

    private GeneralizationNode buildNodeFromThis() {
        GeneralizationNode generalizationNode = new GeneralizationNode(null, name);
        generalizationNode.add(this.generalizationNodes);

        return generalizationNode;
    }
}
