package org.privas.parser;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

public class GeneralizationRange extends Generalization {
    private List<Integer> values;
    private Map<Integer, Integer> positions;

    public GeneralizationRange(int[] numbers) {
        super("*");
        init();

        this.add(numbers);
    }

    public GeneralizationRange(LocalDateTime[] dates) {
        super("*");
        init();

        this.add(dates);
    }

    public GeneralizationRange(String name, String alias) {
        super(name, alias);
        init();
    }

    private void init() {
        this.values = new ArrayList<>();
        this.positions = new HashMap<>();
    }

    @Override
    public boolean isRange() {
        return true;
    }

    public void add(int[] numbers) {
        values = Arrays.stream(numbers).sorted().distinct().boxed().collect(Collectors.toList());

        for (int i = 0; i < values.size(); i++) {
            positions.put(values.get(i), i);
        }
    }

    public void add(LocalDateTime[] dates) {
        values = Arrays.stream(dates).map(date -> date.toInstant(ZoneOffset.UTC).getEpochSecond())
                .map(i -> (int) (long) i)
                .sorted()
                .distinct()
                .collect(Collectors.toList());

        for (int i = 0; i < values.size(); i++) {
            positions.put(values.get(i), i);
        }
    }

    public int getRange() {
        return values.get(values.size() - 1) - values.get(0);
    }

    public int getMaxValue() {
        return values.get(values.size() - 1);
    }

    public boolean isBeforeOrEqual(String valueRaw, int position) {
        return getValuePosition(valueRaw) <= position;
    }

    public int getValuePosition(int value) {
        return positions.get(value);
    }

    public int getValuePosition(String valueRaw) {
        int value = Integer.parseInt(valueRaw);

        return positions.get(value);
    }

    @Override
    public int getNrGeneralizationsAvailable() {
        return 2;
    }
}
