package org.privas.parser;

import java.util.*;
import java.util.stream.Collectors;

public class GeneralizationString extends GeneralizationRange {

    private List<String> values;
    private Map<String, Integer> positions;
    private Map<Integer, String> positionsToValue;

    public GeneralizationString(Collection<String> values) {
        super("*", null);
        init();
        add(values);
    }

    public GeneralizationString(String name, String alias) {
        super("*", alias);
        init();
    }

    private void init() {
        this.values = new ArrayList<>();
        this.positions = new HashMap<>();
        this.positionsToValue = new HashMap<>();
    }

    public void add(Collection<String> values) {
        this.values = values.stream().sorted().distinct().collect(Collectors.toList());

        for (int i = 0; i < this.values.size(); i++) {
            positions.put(this.values.get(i), i);
            positionsToValue.put(i, this.values.get(i));
        }
    }

    @Override
    public int getRange() {
        return positions.get(values.get(values.size() - 1)) - positions.get(values.get(0));
    }

    @Override
    public int getMaxValue() {
        return positions.get(values.get(values.size() - 1));
    }

    @Override
    public boolean isBeforeOrEqual(String value, int position) {
        return positions.get(value) <= position;
    }

    @Override
    public int getValuePosition(int value) {
        return value;
    }

    @Override
    public int getValuePosition(String value) {
        return positions.get(value);
    }
}
