package org.privas.parser;

import org.privas.data.RepositoryType;
import org.privas.techniques.AttackModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.privas.parser.ParserErrorType.*;

public class PrivacyDefinition {
    private String connectionString;
    private String path;
    private List<TableToPrivateDetails> tableToPrivateDetails;
    private String name;
    private List<AttackModel> attackModels;
    private Map<String, Generalization> generalizations;
    private List<ParserError> parserErrors;

    private RepositoryType repositoryType;

    public PrivacyDefinition() {
        this.generalizations = new HashMap<>();
        this.parserErrors = new ArrayList<>();
    }

    public List<TableToPrivateDetails> getTableToPrivateDetails() {
        return tableToPrivateDetails;
    }

    public void setTableToPrivateDetails(List<TableToPrivateDetails> tableToPrivateDetails) {
        this.tableToPrivateDetails = tableToPrivateDetails;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getConnectionString() {
        return connectionString;
    }

    public void setConnectionString(String connectionString) {
        this.connectionString = connectionString;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;

        if (this.path != null) {
            this.path = this.path.substring(1, this.path.length() - 1);
        }
    }

    public RepositoryType getRepositoryType() {
        return repositoryType;
    }

    public void setRepositoryType(RepositoryType repositoryType) {
        this.repositoryType = repositoryType;
    }

    public void setAttackModels(List<AttackModel> attackModels) {
        this.attackModels = attackModels;
    }

    public boolean hasErrors() {
        return parserErrors.size() > 0;
    }

    public int getType() {

        if (attackModels.contains(AttackModel.ATTRIBUTE_LINKAGE) &&  attackModels.contains(AttackModel.TABLE_LINKAGE)) {
            return 4;
        }
        else if (attackModels.contains(AttackModel.ATTRIBUTE_LINKAGE) &&  attackModels.contains(AttackModel.PROBABILISTIC_ATTACK)) {
            return 3;
        }
        else if (attackModels.contains(AttackModel.RECORD_LINKAGE) &&  attackModels.contains(AttackModel.ATTRIBUTE_LINKAGE)) {
            return 2;
        }
        else if (attackModels.contains(AttackModel.RECORD_LINKAGE)) {
            return 1;
        }

        return 0;
    }

    public Map<String, Generalization> getGeneralizations() {
        return generalizations;
    }

    public void addGeneralization(Generalization generalization) {
        if (this.generalizations.containsKey(generalization.getName())) {
            parserErrors.add(new ParserError(generalization.getName(), GENERALIZATION_NAME_DUPLICATED));
        }

        this.generalizations.put(generalization.getName(), generalization);

        if (generalization.getAlias() != null) {
            if (this.generalizations.containsKey(generalization.getAlias())) {
                parserErrors.add(new ParserError(generalization.getName(), GENERALIZATION_ALIAS_DUPLICATED));
            }

            this.generalizations.put(generalization.getAlias(), generalization);
        }
    }

    public List<String> getErrors() {
        return ParserErrorMapper.mapErrors(parserErrors);
    }

    public void calculateErrors() {
        if (attackModels.isEmpty()) {
            parserErrors.add(new ParserError(MISSING_ATTACK_MODELS));
        }

        if (this.getType() == 0) {
            String errorValue = attackModels.stream().map(AttackModel::toString).collect(Collectors.joining("+"));

            parserErrors.add(new ParserError(errorValue, INVALID_ATTACK_MODELS_COMBINATION));
        }
    }
}
