package org.privas.techniques;

import org.privas.parser.Generalization;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class CurrentGeneralizations {

    private int nrTotalElements;
    private List<CurrentGeneralization> currentGeneralizations;
    private int nrIdsGeneralized;

    public CurrentGeneralizations(int nrTotalElements) {
        currentGeneralizations = new ArrayList<>();
        this.nrTotalElements = nrTotalElements;
        this.nrIdsGeneralized = 0;
    }

    CurrentGeneralizations(CurrentGeneralizations currentGeneralizations) {
        this.currentGeneralizations = currentGeneralizations.getCurrentGeneralizations().stream()
                .map(CurrentGeneralization::clone)
                .collect(Collectors.toList());
        this.nrTotalElements = currentGeneralizations.getNrTotalElements();
        this.nrIdsGeneralized = currentGeneralizations.getNrIdsGeneralized();
    }

    public CurrentGeneralization get(int pos) {
        return currentGeneralizations.get(pos);
    }

    public void add(Generalization generalization) {
        currentGeneralizations.add(new CurrentGeneralization(generalization.getName(), generalization.getNrGeneralizationsAvailable()));
    }

    public void setGeneralization(int pos, String value) {
        CurrentGeneralization currentGeneralization = this.currentGeneralizations.get(pos);

        this.currentGeneralizations.set(pos, currentGeneralization.setValue(value));
    }

    public int getNrTotalElements() {
        return nrTotalElements;
    }

    public int getNrIdsGeneralized() {
        return nrIdsGeneralized;
    }

    public void addIdGeneralized() {
        this.nrIdsGeneralized++;
    }

    public List<String> getGeneralizationValues() {
        return this.currentGeneralizations.stream().map(CurrentGeneralization::getValue).collect(Collectors.toList());
    }

    public double getInformationLossPerElement() {
        return 100d / nrTotalElements;
    }

    public double getInformationLossIdElements() {
        return getInformationLossPerElement() * nrIdsGeneralized;
    }

    public double getInformationLossElementsFromCurrentGeneralization() {
        return currentGeneralizations.stream()
                .map(CurrentGeneralization::getInformationLossRatio)
                .mapToDouble(ratio -> ratio * getInformationLossPerElement())
                .sum();
    }

    @Override
    protected CurrentGeneralizations clone() {
        return new CurrentGeneralizations(this);
    }

    List<CurrentGeneralization> getCurrentGeneralizations() {
        return currentGeneralizations;
    }
}
