package org.privas.techniques;

import java.util.ArrayList;
import java.util.List;

public class Partition {

    /*Class for Group, which is used to keep records
    Store tree node in instances.
    self.member: records in group
    self.width: width of this partition on each domain
    self.middle: save the generalization result of this partition
    self.allow: 0 donate that not allow to split, 1 donate can be split
    */

    private List<List<String>> records;
    private List<Tuple<Integer, Integer>> width;
    private CurrentGeneralizations currentQIdsGeneralization;
    private List<Boolean> allow;
    private int numberOfQIds;

    public Partition(List<List<String>> data, List<Tuple<Integer, Integer>> width, CurrentGeneralizations currentQIdsGeneralization, int numberOfQIds) {
        /*
        initialize with data, width and middle
        */
        this.records = new ArrayList<>(data);
        this.width = new ArrayList<>(width);
        this.currentQIdsGeneralization = currentQIdsGeneralization.clone();
        this.numberOfQIds = numberOfQIds;

        this.allow = new ArrayList<>();
        for (int i = 0; i < numberOfQIds; i++) {
            this.allow.add(true);
        }
    }

    public int size() {
        /*
        return the number of records in partition
        */

        return this.records.size();
    }

    public List<List<String>> getRecords() {
        return records;
    }

    public List<Tuple<Integer, Integer>> getWidth() {
        return width;
    }

    public CurrentGeneralizations getCurrentQIdsGeneralization() {
        return currentQIdsGeneralization;
    }

    public List<String> getCurrentQIdsGeneralizationValues() {
        return currentQIdsGeneralization.getGeneralizationValues();
    }

    public boolean isAllowed(int index) {
        return allow.get(index);
    }

    public void disallow(int index) {
        allow.set(index, false);
    }

    public boolean anyAllowed() {
        return allow.parallelStream().anyMatch(e -> e);
    }

    public void totallyAnonymize(int index) {
        for (List<String> row : records) {
            row.set(index, "*");
        }

        currentQIdsGeneralization.addIdGeneralized();
    }

    public List<String> getNotQIds(int indexRow) {
        List<String> notQIds = new ArrayList<>();

        List<String> row = records.get(indexRow);

        for (int i = numberOfQIds; i < row.size(); i++) {
            notQIds.add(row.get(i));
        }

        return notQIds;
    }

    public double getInformationLossPercentage() {
        double informationLossIdsPerRow = currentQIdsGeneralization.getInformationLossIdElements();
        double informationLossQIdsPerRow = currentQIdsGeneralization.getInformationLossElementsFromCurrentGeneralization();

        double informationLossIds = informationLossIdsPerRow * records.size();
        double informationLossQIds = informationLossQIdsPerRow * records.size();

        return informationLossIds + informationLossQIds;
    }
}
